/*******************************************************************************
*	FUNCTION:		Auto_Ball.c
*	PURPOSE:		Go for the trackballs
*                   We have seven possible Ball moves
*   GLOBALS: fieldPosition 1, 2, or 3
*
*   moving gyro turn w/o ball from 0: 65=20deg, -150=-45deg, -300=-85deg, -400=-120deg
*   stopped gyro turn w/ball from 0: 450=57.9 to 72.2
*   stopped gyro turn w/o ball from 0: 450=48.7deg, -450=-70.8deg
*
*   Encoder distance: 36=49", 180=210", 228=253", (?106=108")
*******************************************************************************/
#include "stdio.h"
#include "ifi_frc.h"
#include "adc.h"
#include "gyro.h"
#include "pwm.h"
#include "timers.h"
#include "interrupts.h"
#include "encoder.h"
#include "serial_ports.h"
#include "ifi_code.h"
#include "autonomous.h"
#include "teleop.h"

#define ARM_FLUTTER 7
#define FLUTTER_DELAY 19
#define ARM_DELAY (20 + FLUTTER_DELAY + ARM_FLUTTER)

char grabber1_toggle=1, auto1_toggle=1, flutter_toggle=1;
char done=START;
int heading, distance;
unsigned int desired_joint=0;

/*******************************************************************************
*	FUNCTION:		Auto_Ball_Straight()
*	PURPOSE:		Trackball is straight ahead
*******************************************************************************/
void Auto_Ball_Straight()
{

  static char state=0;
  static int counter=0;
  static int debugtime=0;

  debugtime++;

  switch(state)
  {
    case 0: // Raise arm
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Raise the arm
        counter++;
        if (counter == 19)
        {
            BALL_TOGGLE; //(open grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 2;
            printf("Auto_Ball_Straight--Raise Arm, %d\r\n",debugtime);
        }
	    break;

    case 1: // unused
        state = 2;
	    break;

    case 2:  // Arm Flutter--close the arms quickly to avoid whacking any robot next to us
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == ARM_FLUTTER)
        {
            BALL_TOGGLE; //(close grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 3;
            printf("Arm Flutter, %d\r\n",debugtime);
        }
        break;

    case 3: // Drive forward
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
#if 1
        if (counter == 38)  // Reopen the grabber after the arm has come up a bit and we've driven somewhat clear
        {
            BALL_TOGGLE; //(open grabber) we'll be grabbing the ball we will
            printf("Open grabber, %d\r\n",debugtime);
        }
#endif
        distance = 250; //238 inches, but we'll get some drift as the robot coasts to a stop
        if (fieldPosition == FIELDRIGHT) distance -= 33; // The rightmost starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            //BALL_TOGGLE; //(close grabber) we'll be grabbing the ball we will
            //printf("Close grabber, %d\r\n",debugtime);
            counter = 0; // reset our time counter
            state = 4;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 4: // Grab ball and start lift
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == 38) // delay a second to complete momentum movement
        {
            BALL_TOGGLE; //(close grabber) hope the ball's in our arms
            printf("Close grabber, %d\r\n",debugtime);
        }
        if (counter == 58) // Wait again to be sure we have the ball
        {
            desired_joint = ARM_CAP;
            arm_movement(desired_joint, CAP); // Start lifting the ball
            done = START;
            counter = 0; // reset our time counter
            state = 5;
            printf("Grab ball & lift, %d\r\n",debugtime);
        }
        break;

    case 5: // Raise ball off overpass
        desired_joint = ARM_CAP;
        arm_movement(desired_joint, CAP);
        counter++;
        if (counter == 30)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 6;
            printf("Raise ball off overpass, %d\r\n",debugtime);
        }
        break;

    case 6: // Back away from the Overpass
        desired_joint = ARM_CAP;
        arm_movement(desired_joint, CAP);
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
            done = START;
            counter = 0; // reset our time counter
            state = 7;
            printf("Back away, %d\r\n",debugtime);
        }
        break;

    case 7: // Back away from the Overpass & bring the ball down
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        Drive_Backward2();
        counter++;
        if (counter == 12)
        {
            done = START;
            Drive_Stop();
            counter = 0; // reset our time counter
            state = 71;
            //state = 99; // Stop here
            printf("Lower ball, %d\r\n",debugtime);
        }
        break;

    case 71: // Give it time to come to a rest
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        Drive_Stop();
        counter++;
        if (counter == 38)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 8;
            //state = 99; // Stop here
            printf("Rest, %d\r\n",debugtime);
        }
        break;

    case 8: // Move on down the track
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        distance = 325; //230 inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 9;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 9: // Turn to face the next quadrant
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = -300;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 10;
            printf("Make the track turn, %d\r\n",debugtime);
        }
        break;

    case 10: // Move on around the track turn
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        if (fieldPosition == FIELDLEFT)   distance = 70; // The left starting position has a shorter drive
        if (fieldPosition == FIELDCENTER) distance = 86; // The center starting position has a shorter drive
        if (fieldPosition == FIELDRIGHT)  distance = 120; // The right starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 99:
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //Done
        break;
  }
}

/*******************************************************************************
*	FUNCTION:		Auto_Ball_Straight_Knockoff()
*	PURPOSE:		Trackball is straight ahead
*******************************************************************************/
void Auto_Ball_Straight_Knockoff()
{

  static char state=0;
  static int counter=0;
  static int debugtime=0;

  debugtime++;

  switch(state)
  {
    case 0: // Raise arm
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Raise the arm
        counter++;
        if (counter == 19)
        {
            BALL_TOGGLE; //(open grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 2;
            printf("Auto_Ball_Straight Knockoff--Raise Arm, %d\r\n",debugtime);
        }
	    break;

    case 1: // unused
        state = 2;
	    break;

    case 2:  // Arm Flutter--close the arms quickly to avoid whacking any robot next to us
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == ARM_FLUTTER)
        {
            BALL_TOGGLE; //(close grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 3;
            printf("Arm Flutter, %d\r\n",debugtime);
        }
        break;

    case 3: // Drive forward
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == 38)  // Reopen the grabber after the arm has come up a bit and we've driven somewhat clear
        {
            PUNCH; //Bring the kicker up
            printf("Kicker up, %d\r\n",debugtime);
        }
        distance = 238; //inches, but we'll get some drift as the robot coasts to a stop
        if (fieldPosition == FIELDRIGHT) distance -= 33; // The rightmost starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 6;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 6: // Back away from the Overpass
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
            done = START;
            counter = 0; // reset our time counter
            state = 7;
            printf("Back away, %d\r\n",debugtime);
        }
        break;

    case 7: // Back away from the Overpass & bring the ball down
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        RETRACT;
        Drive_Backward2();
        counter++;
        if (counter == 12)
        {
            done = START;
            Drive_Stop();
            counter = 0; // reset our time counter
            state = 71;
            //state = 99; // Stop here
            printf("Lower ball, %d\r\n",debugtime);
        }
        break;

    case 71: // Give it time to come to a rest
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        Drive_Stop();
        counter++;
        if (counter == 38)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 8;
            //state = 99; // Stop here
            printf("Rest, %d\r\n",debugtime);
        }
        break;

    case 8: // Move on down the track
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        distance = 230; //inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 9: // Turn to face the next quadrant
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = -300;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 10;
            printf("Make the track turn, %d\r\n",debugtime);
        }
        break;

    case 10: // Move on around the track turn
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        if (fieldPosition == FIELDLEFT)   distance = 20; // The left starting position has a shorter drive
        if (fieldPosition == FIELDCENTER) distance = 20; // The center starting position has a shorter drive
        if (fieldPosition == FIELDRIGHT)  distance = 20; // The right starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 99:
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //Done
        break;
  }
}


/*******************************************************************************
*	FUNCTION:		Auto_Ball_LR_1()
*	PURPOSE:		Trackball is one position to the left or right
*   INPUTS:    side- left or right position TO_THE_LEFT or TO_THE_RIGHT
*******************************************************************************/
void Auto_Ball_LR_1(char side)
{
  static char state=0;
  static int counter=0;
  static int debugtime=0;

  debugtime++;

  switch(state)
  {
    case 0: // Raise arm
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Raise the arm
        counter++;
        if (counter == 19)
        {
            BALL_TOGGLE; //(open grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 2;
            printf("Auto_Ball_Straight--Raise Arm, %d\r\n",debugtime);
        }
	    break;

    case 1: // unused
        state = 2;
	    break;

    case 2:  // Arm Flutter--close the arms quickly to avoid whacking any robot next to us
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == ARM_FLUTTER)
        {
            BALL_TOGGLE; //(close grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 3;
            printf("Arm Flutter, %d\r\n",debugtime);
        }
        break;

    case 3: // Drive forward
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
#if 0
        counter++;
        if (counter == 38)  // Bring up the kicker after the arm has come up a bit and we've driven somewhat clear
        {
            PUNCH;
            printf("Kicker up, %d\r\n",debugtime);
        }
#endif
        distance = 106; //inches, but we'll get some drift as the robot coasts to a stop
        if (fieldPosition == FIELDRIGHT) distance -= 33; // The rightmost starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            PUNCH;
            counter = 0; // reset our time counter
            state = 31;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 31: // Turn to the side
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        // Headings from 0 -60 = 20deg, -150=-45 deg, -300=-85, -400=-120
        if(side == TO_THE_LEFT)
            heading = -60;
        else
            heading = 65;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            Drive_Stop();
            done = START;
            counter = 0; // reset our time counter
            state = 311;
            printf("Gyro turn %d, %d\r\n",heading, debugtime);
        }
        break;

    case 311: // Delay so robot can rest for the encoders
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Keep the arm raised
        Drive_Stop();
        counter++;
        if (counter == 38) // Wait again to be sure we have the ball
        {
            done = START;
            counter = 0; // reset our time counter
            state = 32;
            printf("Delay, %d\r\n",debugtime);
        }
        break;

    case 32: // Drive forward again
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        distance = 103; //inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 4;
            printf("Drive Forward Again=, %d\r\n",debugtime);
        }
        break;

    case 4: // Delay so robot can drift into the trackball
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Keep the arm raised
        counter++;
        if (counter == 38) // Wait again to be sure we have the ball
        {
            done = START;
            counter = 0; // reset our time counter
            state = 6;
            printf("Drift into trackball, %d\r\n",debugtime);
        }
        break;

    case 5: // Unused
        state = 6;
        break;

    case 6: // Back away from the Overpass
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Keep the arm raised
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
            RETRACT;
            done = START;
            counter = 0; // reset our time counter
            state = 7;
            printf("Back away, %d\r\n",debugtime);
        }
        break;

    case 7: // Back away from the Overpass & bring the arm down
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            done = START;
            Drive_Stop();
            counter = 0; // reset our time counter
            state = 71;
            //state = 99; // Stop here
            printf("Lower ball, %d\r\n",debugtime);
        }
        break;

    case 71: // Turn to face straight again
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = 0;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 72;
            printf("Straight up the field again, %d\r\n",debugtime);
        }
        break;

    case 72: // Delay so robot can drift into the trackball
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        counter++;
        if (counter == 38) // Wait again to be sure we have the ball
        {
            done = START;
            counter = 0; // reset our time counter
            state = 8;
            printf("Delay, %d\r\n",debugtime);
        }
        break;

    case 8: // Move on down the track
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //distance = 230; //inches, but we'll get some drift as the robot coasts to a stop
        distance = 72; //inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            //state = 9;
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 9: // Turn to face the next quadrant
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = -300;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 10;
            printf("Make the track turn, %d\r\n",debugtime);
        }
        break;

    case 10: // Move on around the track turn
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        if (fieldPosition == FIELDLEFT)   distance = 20; // The left starting position has a shorter drive
        if (fieldPosition == FIELDCENTER) distance = 20; // The center starting position has a shorter drive
        if (fieldPosition == FIELDRIGHT)  distance = 20; // The right starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 99:
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //Done
        break;
  }
}


/*******************************************************************************
*	FUNCTION:		Auto_Ball_LR_2()
*	PURPOSE:		Trackball is two positions to the left or right
*                   Only happens from the right and left most starting positions
*   INPUTS:  side - left or right position
*******************************************************************************/
void Auto_Ball_LR_2(char side)
{
  static char state=0;
  static int counter=0;
  static int debugtime=0;

  debugtime++;

  switch(state)
  {
    case 0: // Raise arm
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Raise the arm
        counter++;
        if (counter == 19)
        {
            BALL_TOGGLE; //(open grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 2;
            printf("Auto_Ball_Straight--Raise Arm, %d\r\n",debugtime);
        }
	    break;

    case 1: // unused
        state = 2;
	    break;

    case 2:  // Arm Flutter--close the arms quickly to avoid whacking any robot next to us
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == ARM_FLUTTER)
        {
            BALL_TOGGLE; //(close grabber)
            done = START;
            counter = 0; // reset our time counter
            state = 3;
            printf("Arm Flutter, %d\r\n",debugtime);
        }
        break;

    case 3: // Drive forward
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        if (counter == 38)  // Bring up the kicker after the arm has come up a bit and we've driven somewhat clear
        {
            PUNCH;
            printf("Open grabber, %d\r\n",debugtime);
        }
        distance = 106; //inches, but we'll get some drift as the robot coasts to a stop
        if (fieldPosition == FIELDRIGHT) distance -= 33; // The rightmost starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 31;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 31: // Turn to the side
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        // Headings from 0 -60 = 20deg, -150=-45 deg, -300=-85, -400=-120
        if(side == TO_THE_LEFT)
            heading = -69; // Just a guess
        else
            heading = 70;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 32;
            printf("Gyro turn, %d\r\n",debugtime);
        }
        break;

    case 32: // Drive forward again
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Continue to raise the arm
        counter++;
        distance = 103; //inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 4;
            printf("Drive Forward Again=, %d\r\n",debugtime);
        }
        break;

    case 4: // Delay so robot can drift into the trackball
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Keep the arm raised
        counter++;
        if (counter == 38) // Wait again to be sure we have the ball
        {
            done = START;
            counter = 0; // reset our time counter
            state = 6;
            printf("Drift into trackball, %d\r\n",debugtime);
        }
        break;

    case 5: // Unused
        state = 6;
        break;

    case 6: // Back away from the Overpass
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF); // Keep the arm raised
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
            done = START;
            counter = 0; // reset our time counter
            state = 7;
            printf("Back away, %d\r\n",debugtime);
        }
        break;

    case 7: // Back away from the Overpass & bring the arm down
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        Drive_Backward2();
        counter++;
        if (counter == 20)
        {
            done = START;
            counter = 0; // reset our time counter
            //state = 71;
            state = 99;
            printf("Lower ball, %d\r\n",debugtime);
        }
        break;

    case 71: // Turn to face straight again
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = 0;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
//            state = 8;
            state = 88; // Stop here unil we see how far off the robot is from the trackball
            printf("Straight up the field again, %d\r\n",debugtime);
        }
        break;

    case 8: // Move on down the track
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        distance = 230; //inches, but we'll get some drift as the robot coasts to a stop
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 9;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 9: // Turn to face the next quadrant
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        heading = -300;
        Gyro_Turn(heading, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 10;
            printf("Make the track turn, %d\r\n",debugtime);
        }
        break;

    case 10: // Move on around the track turn
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        if (fieldPosition == FIELDLEFT)   distance = 20; // The left starting position has a shorter drive
        if (fieldPosition == FIELDCENTER) distance = 20; // The center starting position has a shorter drive
        if (fieldPosition == FIELDRIGHT)  distance = 20; // The right starting position has a shorter drive
        Drive_Forward(distance, &done);
        if (done == COMPLETED)
        {
            done = START;
            counter = 0; // reset our time counter
            state = 99;
            printf("Drive Forward, %d\r\n",debugtime);
        }
        break;

    case 88:
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //Done
        break;

    case 99:
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        //Done
        break;
  }
}


#if 0
#define ARM_FLUTTER 7
#define FLUTTER_DELAY 19
#define ARM_DELAY (20 + FLUTTER_DELAY + ARM_FLUTTER)
//DEBUG test encoder driving
if (p2_sw_aux1 && auto1_toggle)
{
    auto1_toggle = 0;
    encoder_drive=TRUE;
    desired_joint = ARM_KNOCKOFF;
    arm_movement(desired_joint, KNOCKOFF);
    wait_counter = ARM_DELAY;
    edone = ARM_WAIT;
    wait_counter3 = FLUTTER_DELAY;
    printf("lift arm\r\n");
    done = START;
}

if (encoder_drive)
{
    desired_joint = ARM_KNOCKOFF;
    arm_movement(desired_joint, KNOCKOFF);
    if (edone == ARM_WAIT)
    {
        if (wait_counter > 0)
            wait_counter--;
        else
        {
            edone=START;
            BALL_TOGGLE;
        }

        if (wait_counter2 > 0)
            wait_counter2--;
        if (wait_counter2 == 1)
        {
            BALL_TOGGLE;
        }

        if (wait_counter3 > 0)
            wait_counter3--;
        if (wait_counter3 == 1)
        {
            wait_counter2 = ARM_FLUTTER;
            BALL_TOGGLE;
        }
    }
    else
    {
        if (edone!=COMPLETED)
        {
            //distance = 238; //Center ball position (straight ahead)
            distance = 106; //Side ball positions (20 degrees)
            Drive_Forward(distance, &edone);
            PUNCH;
            if (edone == COMPLETED)
            {
                e2done = GYROTURN;
                //done = START;
                timed_grab = 0;
            }
        }
        else if (e2done==GYROTURN)
        {
            if (done!=COMPLETED)
            {
                heading = 65; // 65 works for right ball, -60 worked for left ball, -50 worked then missed (-100 = 80 degrees)
                Gyro_Turn(heading, &done);
                e3done=START;
            }
            else
            {
                //edone=DRIVE_SOME_MORE;
                //e2done=START;
                distance = 103; //Side ball positions (20 degrees)
                Drive_Forward(distance, &e3done);
                if (e3done == COMPLETED)
        
        {
          timed_grab++;
          if (timed_grab == 38)
          {
              BALL_TOGGLE;
          }
          if (timed_grab > 58)
          {
              desired_joint = ARM_CAP;
              arm_movement(desired_joint, CAP);
          }
          if (timed_grab > 88 && timed_grab < 108)
          {
            Drive_Backward2();
          }
          if (timed_grab >= 108 && timed_grab < 120)
          {
            Drive_Backward2();
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
            RETRACT;
          }
          if (timed_grab >= 120)
          {
            if (done!=COMPLETED)
            {
                heading = 0;
                Gyro_Turn(heading, &done);
            }
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
          }
          if (timed_grab > 184 && timed_grab < 222)
          {
            Drive_Forward2();
          }
        }
      }  
    }
  }  
}
// End DEBUG
#endif
