/*******************************************************************************
*	FUNCTION:		Auto_Functions.c
*	PURPOSE:		Miscellaneous Auotnomous/Hybrid functions
*******************************************************************************/
#include "stdio.h"
#include "ifi_frc.h"
#include "autonomous.h"
#include "teleop.h"
#include "encoder.h"

/*******************************************************************************
*	FUNCTION:		Redirect_Automode()
*	PURPOSE:		Change the Autonomous we're running based on the IR command
*******************************************************************************/
void Redirect_Automode (char side)
{
    if (IR_cmd_tail != IR_cmd_index) // We have an IR command to act on
    {
        switch(IR_cmd_array[IR_cmd_index])
        {
            case 1:
                if (side == NEARSIDE)
                {
                    autoMode = 1; // Near Ball 1
                }
                else
                {
                    autoMode = 5; // Far Ball 1
                }
                break;

            case 2:
                if (side == NEARSIDE)
                {
                    autoMode = 2; // Near Ball 2
                }
                else
                {
                    autoMode = 6; // Far Ball 2
                }
                break;

            case 3:
                if (side == NEARSIDE)
                {
                    autoMode = 3; // Near Ball 3
                }
                else
                {
                    autoMode = 7; // Far Ball 3
                }
                break;

            case 4:
                autoMode = 4; // Race
                break;

            default:
                autoMode = DEFAULT_AUTO; //Default auto
        }
    }
    else
    {
        autoMode = DEFAULT_AUTO;
    }
}


/*******************************************************************************
*	FUNCTION:		IR_Receive()
*	PURPOSE:		Check for IR command(s)
*******************************************************************************/
void IR_Receive(char *IR_cmd)
{
  unsigned char sensorReading;

  //**** Check the IR Sensor for a new command
  sensorReading  = PORTB>>4;  // Combined digital inputs 3-6

  if (sensorReading != 0)
  {
	   *IR_cmd=0;
       if (sensorReading == 8)      *IR_cmd = 3; //3
       else if (sensorReading == 4) *IR_cmd = 2; //2
       else if (sensorReading == 2) *IR_cmd = 4; //4
       else if (sensorReading == 1) *IR_cmd = 1; //1
       else 
         *IR_cmd = 99;  //Error condition, IF failure

        //printf("IR_cmd = %d\r\n", *IR_cmd);
  }
}


/*******************************************************************************
* FUNCTION NAME: Drive_Forward
* PURPOSE:       Encoder based
* ARGUMENTS:     input: distance in inches                
* AndyMark encoders
* 13.25555 ticks per inch
* Forward 180": E1=2386, E2=2374
* Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)
*******************************************************************************/
#define MAX_SPEED   100
//#define MAX_SPEED   30
#define MAX_ERROR   80
#define ENCODER_P   26/100
#define RKP 10/10
#define LKP 10/10

void Drive_Forward(int distance, char *done)
{
  static long our_distance=0;
  long drive, error, driveR, driveL;
  long Encoder_Count, Right_Encoder_Count, Left_Encoder_Count;
  unsigned char wheel;

  if (*done == START)
  {
    *done == IN_PROGRESS;
    our_distance = (long)distance * 2386 / 180; 
    Reset_Encoder_1_Count();
    Reset_Encoder_2_Count();
    printf("Encoder start: our_distance=%ld\r\n", our_distance);
  }

  Left_Encoder_Count = Get_Encoder_1_Count();
  Right_Encoder_Count = Get_Encoder_2_Count();
  Encoder_Count = (Right_Encoder_Count + Left_Encoder_Count)/2;

  if (Encoder_Count >= our_distance)
  {
    *done = COMPLETED;
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
    printf("Drive Forward COMPLETED: LeftE=%ld, RightE=%ld\r\n", Left_Encoder_Count, Right_Encoder_Count);
  }
  else
  {
    *done = IN_PROGRESS;
    drive = (our_distance - Encoder_Count) * ENCODER_P;
    if (drive > MAX_SPEED) drive = MAX_SPEED; //Leave room to balance each side
    if (drive < 0) drive = 0;

    //wheel = p3_wheel>>2;
    wheel = 20; // Either 10 or 20 works fine

    //Balance the drives
    error = (Right_Encoder_Count - Left_Encoder_Count) * wheel/10;
    if (error > MAX_ERROR) error = MAX_ERROR; //Don't allow it to stop altogether

    driveR = 127 + drive - error;
    if (driveR > 254) driveR = 254;
    if (driveR < 127) driveR = 127;
    driveL =  127 + drive + error;
    if (driveL > 254) driveL = 254;
    if (driveL < 127) driveL = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = driveR;
    LEFT_MOTOR_2 = LEFT_MOTOR_1 = driveL;
    printf("RE=,%ld, LE=,%ld, error=,%ld, pid=,%ld, Rm=,%d, Lm=,%d, KP=%d/10,\r\n", 
            Right_Encoder_Count, Left_Encoder_Count, error, drive, RIGHT_MOTOR_1, LEFT_MOTOR_1, wheel);
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward
* PURPOSE:       Encoder based
* ARGUMENTS:     input: distance in inches                
* AndyMark encoders
* 13.25555 ticks per inch
* Forward 180": E1=2386, E2=2374
* Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)
* One encoder is broken and only produces a count not +/- direction
*  UNTESTED
*******************************************************************************/

void Drive_Backward(int distance, char *done)
{
  static long our_distance=0;
  long drive, error;
  long Encoder_Count, Right_Encoder_Count, Left_Encoder_Count;
  unsigned char wheel;

  if (*done == START)
  {
    *done == IN_PROGRESS;
    our_distance = (long)distance * -2330 / 180; 
    Reset_Encoder_1_Count();
    Reset_Encoder_2_Count();
    printf("Encoder start: our_distance=%ld\r\n", our_distance);
  }

  Left_Encoder_Count = Get_Encoder_1_Count() * -1; // Broken encoder B channel
  Right_Encoder_Count = Get_Encoder_2_Count();
  Encoder_Count = (Right_Encoder_Count + Left_Encoder_Count)/2;

  if (Encoder_Count <= our_distance)
  {
    *done = COMPLETED;
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
    printf("Drive Forward COMPLETED: LeftE=%ld, RightE=%ld\r\n", Left_Encoder_Count, Right_Encoder_Count);
  }
  else
  {
    *done = IN_PROGRESS;
    drive = (our_distance - Encoder_Count) * ENCODER_P;
    if (drive > MAX_SPEED) drive = MAX_SPEED; //Leave room to balance each side
    if (drive < 0) drive = 0;  

    //wheel = p3_wheel>>2;
    wheel = 20; // Either 10 or 20 works fine

    //Balance the drives
        error = (Right_Encoder_Count - Left_Encoder_Count) * wheel/10;
        if (error > MAX_ERROR) error = MAX_ERROR; //Don't allow it to stop altogether
        RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127 + drive - error;
        LEFT_MOTOR_2 = LEFT_MOTOR_1 = 127 + drive + error;
        //printf("RE=,%ld, LE=,%ld, error=,%ld, pid=,%ld, Rm=,%d, Lm=,%d, KP=%d/10,\r\n", 
        //    Right_Encoder_Count, Left_Encoder_Count, error, drive, RIGHT_MOTOR_1, LEFT_MOTOR_1, wheel);
  }
}


/*******************************************************************************
* FUNCTION NAME: Drive_Arc
* PURPOSE:       Encoder based
* ARGUMENTS:     input: distance in inches                
* AndyMark encoders
* 13.25555 ticks per inch
* Forward 180": E1=2386, E2=2374
* Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)
*    28" wheelbase, typical arc 48" diameter inside radius, 104" diameter outside
*    circumference = diameter * 3.14159
*    right side full speed, left side moderated to keep pace
*    total left ticks = 2380*48"*3.14
*    total right ticks = 2380*(48+2*28)*3.14
*******************************************************************************/

void Drive_Arc(int inside_diameter, int arc, char *done)
{
  static long inside_distance=0, outside_distance=0;
  long drive, error;
  long Encoder_Count, Right_Encoder_Count, Left_Encoder_Count;
  unsigned char wheel;

  if (*done == START)
  {
    *done == IN_PROGRESS;
    inside_distance = (long)inside_diameter * 314; //increase accuracy
    inside_distance = inside_distance * 2380 / 180; 
    Reset_Encoder_1_Count();
    Reset_Encoder_2_Count();
    printf("Encoder start: inside_distance=%ld\r\n", inside_distance);
  }

  Left_Encoder_Count = Get_Encoder_1_Count();
  Right_Encoder_Count = Get_Encoder_2_Count();
  Encoder_Count = (Right_Encoder_Count + Left_Encoder_Count)/2;

  if (Encoder_Count >= inside_distance)
  {
    *done = COMPLETED;
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
    printf("Drive Arc COMPLETED: LeftE=%ld, RightE=%ld\r\n", Left_Encoder_Count, Right_Encoder_Count);
  }
  else
  {
    *done = IN_PROGRESS;
    drive = (inside_distance - Encoder_Count) * ENCODER_P;
    if (drive > MAX_SPEED) drive = MAX_SPEED; //Leave room to balance each side
    if (drive < 0) drive = 0;  

    wheel = 20; // Either 10 or 20 works fine

    //Balance the drives
        error = (Right_Encoder_Count - Left_Encoder_Count) * wheel/10;
        if (error > MAX_ERROR) error = MAX_ERROR; //Don't allow it to stop altogether
        RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127 + drive - error;
        LEFT_MOTOR_2 = LEFT_MOTOR_1 = 127 + drive + error;
        printf("RE=,%ld, LE=,%ld, error=,%ld, pid=,%ld, Rm=,%d, Lm=,%d, KP=%d/10,\r\n", 
            Right_Encoder_Count, Left_Encoder_Count, error, drive, RIGHT_MOTOR_1, LEFT_MOTOR_1, wheel);
  }
}


/*******************************************************************************
* FUNCTION NAME: Drive_Backward2
* PURPOSE:       Encoder based
* ARGUMENTS:     input: distance in inches                
* AndyMark encoders
* 13.25555 ticks per inch
* Forward 180": E1=2386, E2=2374
* Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)
*******************************************************************************/

void Drive_Backward2(void)
{
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 0;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 0;
}

/*******************************************************************************
* FUNCTION NAME: Drive_Forward2
* PURPOSE:       Timer based
* ARGUMENTS:     input: distance in inches                
* AndyMark encoders
* 13.25555 ticks per inch
* Forward 180": E1=2386, E2=2374
* Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)
*******************************************************************************/

void Drive_Forward2(void)
{
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 254;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 254;
}


/*******************************************************************************
* FUNCTION NAME: Drive_Stop
* PURPOSE:       Enough is enough
*******************************************************************************/

void Drive_Stop(void)
{
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
}
