/*******************************************************************************
*
*	TITLE:		autonomous.c 
*
*	VERSION:	0.1 (Beta)                           
*
*	DATE:		31-Dec-2007
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This file best viewed with tabs set to four.
*
*				You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or elsewhere
*				without permission. Thanks.
*
*				Copyright 2007-2008 R. Kevin Watson. All rights are reserved.
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	31-Dec-2007  0.1  RKW Original
*
*******************************************************************************/

/////////////////////////////////////////////////////////
// Auto_0: Do Nothing
// Auto_1: Ball position 1
// Auto_2: Ball position 2
// Auto_3: Ball position 3
// Auto_4: Race only
// Auto_5: Far Ball position 1
// Auto_6: Far Ball position 2
// Auto_7: Far Ball position 3
// Auto_8: Near-side IR commanded Auto
// Auto_9: Far-side IR commanded Auto
/////////////////////////////////////////////////////////
// Field position 1: robot lined up with ball 1
// Field position 2: robot lined up with ball 2
// Field position 3: robot lined up with ball 3
/////////////////////////////////////////////////////////

#include "ifi_frc.h"
#include "pwm.h"
#include "gyro.h"
#include "timers.h"
#include "interrupts.h"
#include "encoder.h"
#include "serial_ports.h"
#include "ifi_code.h"
#include "autonomous.h"
#include "teleop.h"

unsigned char autoMode=0, fieldPosition=1;
char IR_cmd_index=0, IR_cmd_tail=0, IR_cmd_array[10]={0,0,0,0,0,0,0,0,0,0};

/*******************************************************************************
*
*	FUNCTION:		Autonomous_Init()
*
*	PURPOSE:		This is where you put code that you want to execute
*					just once at the start of autonomous mode.
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is invalid
*					(i.e., all analog inputs are set to 127 and digital inputs
*					are set to zero) and all robot controller outputs (PWMs and
*					relays) are	enabled.
*
*******************************************************************************/
void Autonomous_Init(void)
{
	// this is where the the gyro bias calculation should end
	Reset_Gyro_Angle();
    SHIFT_HIGH;
    IR_ON;
    autoMode  = ~PORTJ>>4;  // Combined digital inputs 15-18
    //printf("raw AutoMode=%d\r\n",(int)autoMode);
    if (autoMode > 9 ) autoMode = 8;
    printf("AutoMode=%d, fieldPosition= ",(int)autoMode);
    if (!rc_dig_in11)
    {
        printf("RIGHT\r\n");
        fieldPosition = FIELDRIGHT; // =1; //digital inputs 11, 12, 13
    }
    else if (!rc_dig_in12)
    {
        printf("CENTER\r\n");
        fieldPosition = FIELDCENTER; // =2; //digital inputs 11, 12, 13
    }
    else
    {
        printf("LEFT\r\n");
        fieldPosition = FIELDLEFT;  // =3; //digital inputs 11, 12, 13
    }
}

/*******************************************************************************
*
*	FUNCTION:		Autonomous()
*
*	PURPOSE:		This is where you put code that you want to execute while
*					your robot is in autonomous mode. While in autonomous mode,
*					this function is called every 26.2ms after new data is 
*					received from the master processor.				
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is invalid
*					(i.e., all analog inputs are set to 127 and digital inputs
*					are set to zero) and all robot controller outputs (PWMs and
*					relays) are	enabled.
*
*******************************************************************************/
void Autonomous(void)
{
  static char state=0, last_IR_cmd=0; 
  static int counter=0;
  char IR_cmd=0;

  // Always listen for new IR commands
  // Won't take notice of repeated commands until another button has been pushed

  IR_Receive(&IR_cmd);
  //if (IR_cmd == 0) last_IR_cmd = IR_cmd; //This will allow repeated commands
  //if (IR_cmd != last_IR_cmd)
  if ((IR_cmd != last_IR_cmd) && (IR_cmd != 0))
  {
     if (IR_cmd == 99)
     {
        last_IR_cmd = IR_cmd;
        printf("IR Receiver is Disconnected\r\n");
     }
     else
     {
        printf("IR command=%d\r\n", IR_cmd);
        last_IR_cmd = IR_cmd;
        if (IR_cmd_tail < 10)
        {
            IR_cmd_array[IR_cmd_tail] = IR_cmd;
            IR_cmd_tail++;
        }
     }
  }

  switch(autoMode)
  {
    case 0:
        Auto_0();
	    break;

    case 1:
        Auto_1();
	    break;

    case 2:
        Auto_2();
	    break;

    case 3:
        Auto_3();
	    break;

    case 4:
        Auto_4();
	    break;

    case 5:
        Auto_5();
	    break;

    case 6:
        Auto_6();
	    break;

    case 7:
        Auto_7();
	    break;

    case 8:
        Auto_8();
	    break;

    case 9:
        Auto_9();
	    break;

  }
}

/*******************************************************************************
*
*	FUNCTION:		Autonomous_Spin()
*
*	PURPOSE:		While in autonomous mode, this function is called
*					continuously between calls to Autonomous().			
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is invalid
*					(i.e., all analog inputs are set to 127 and digital inputs
*					are set to zero) and all robot controller outputs (PWMs and
*					relays) are	enabled.
*
*******************************************************************************/
void Autonomous_Spin(void)
{
	// enable this function if you want to process gyro data during autonomous mode
	 Process_Gyro_Data();
}
