#include "ifi_frc.h"
#include "adc.h"
#include "gyro.h"
#include "pwm.h"
#include "timers.h"
#include "interrupts.h"
#include "encoder.h"
#include "serial_ports.h"
#include "ifi_code.h"
#include "teleop.h"

int Gyro_Turn_PID(int, long, char);

/*******************************************************************************
* FUNCTION NAME: Gyro_Turn
* PURPOSE:       Turn the robot to the specified heading.
* ARGUMENTS:     input: speed 0-127
*
*                A 90 degree turn will center the back edge where the old back edge was 
*                and will lineup with the old left side. This may vary depending on tire traction.
*                 
*******************************************************************************/
#define GYROWINDOW 15 // tenths of degrees - Considered complete at gyro +/- this window of accuracy
#define LESS 1
#define MORE 2

void Gyro_Turn(int heading, char *done)
{
  int gyrodegrees, pid;
  static int our_heading=1;
  long gyro_angle, drive;
  static char direction;

  gyro_angle = Get_Gyro_Angle();
  //printf("Current Gyro Angle=%d\r\n", (int)gyro_angle);
  if (*done == START)
  {
    if (GyroStatus == NOGYRO) // Gyro did not initialize properly so we can't depend on it
    {
        // Call a timed turn instead so we're not completely dead in the water
        *done = COMPLETED;
        LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
        RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
        printf("****** WARNING GYRO is DISCONNECTED **************\r\n");
    }
    else
    {
        *done == IN_PROGRESS;
        our_heading = heading; //For positive=clockwise
        Gyro_Turn_PID(our_heading,gyro_angle,TRUE);
        if (our_heading < gyro_angle)
            direction = LESS;
        else
            direction = MORE;
        //printf("Gyro start: our_heading=%d\r\n", our_heading);
    }
  }

  if ((direction == LESS) && (gyro_angle <= our_heading))
  {
    *done = COMPLETED;
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
    //printf("Gyro COMPLETED\r\n");
  }
  else if ((direction == MORE) && (gyro_angle >= our_heading))
  {
    *done = COMPLETED;
    LEFT_MOTOR_1 = LEFT_MOTOR_2 = 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = 127;
    //printf("Gyro COMPLETED\r\n");
  }
  else
  {
    *done = IN_PROGRESS;
    pid = Gyro_Turn_PID(our_heading,gyro_angle,FALSE);
    drive = (long)127 + pid;
    if (drive > 254) drive = 254;
    if (drive < 0) drive = 0;
    LEFT_MOTOR_2 = LEFT_MOTOR_1 = 254 - drive; //Spin in place
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = drive;
    //printf("Gyro IN_PROGRESS: pid=%d\r\n", pid);
  }
}

/*******************************************************************************
* FUNCTION NAME: Gyro_Steering
* PURPOSE:       
* CALLED FROM:   various
*   gyro correction, IR correction, encoder correction, regular driving
*   Max free wheeling speed of encoders is ~2575 counts/sec or ~20.1 fps
*   Max under-load speed is ~13 fps
*******************************************************************************/
#define SKP 40/10  // Basic proportional adjustment (40/10 works)
#define SKI 8  // Cumulative error increases adjustment (8 works)
#define SKD 0  // Sudden reaction to sudden changes (use 0)
#define ST  40 // This is executed at the slow loop rate of 40 Hz
#define SCUMLIMIT 5000  // Limit to error accumulation (~ 2 complete robot 360's)
int Gyro_Steering(int expected, int actual, int *LastActual, int *CumError)
{
  int p, i, d, pid, error;

    /* Proportional */
    /* Add correction proportional to the current error */
    error = (expected - actual);
    p = error * (int)SKP;

    /* Integral */
    /* Correct accumulated error */
    i = *CumError * (int)SKI / (int)ST; // p covers current error

    /* Check for runaway accumulated error */
    *CumError += error; 
    if (*CumError > (int)SCUMLIMIT) // Detect and avoid integral windup
      *CumError = (int)SCUMLIMIT;
    if (*CumError < (int)-SCUMLIMIT)// Detect and avoid integral windup
      *CumError = (int)-SCUMLIMIT;

    /* Derivative */
    /* Sudden reaction to sudden changes in the error rate */
    d = (error - *LastActual) * (int)SKD;
    *LastActual = error;

//    pid = p;
    pid = p + i;
//    pid = p + i - d;
  return pid;
}

/*******************************************************************************
* FUNCTION NAME: Gyro_Turn_PID
* PURPOSE:       
*   gyro correction, IR correction, encoder correction, regular driving
*******************************************************************************/
#define GKP 3/10     // Basic proportional adjustment
#define GKI 4         // Cumulative error increases adjustment
#define GKD 0         // Sudden reaction to sudden changes (0)
#define GT  40        // This is executed at the slow loop rate of 40 Hz
#define GCUMLIMIT  127*GT/GKI // Limit to error accumulation
#define STARTMIN   8  // Number of cycles required to get out of the start motor stall
int Gyro_Turn_PID(int expected, long actual, char init)
{
  int p, i, d, pid, error;
  static int LastActual, CumError;
  static char Start, StartCount, Positive;

    if (init==TRUE)
    {
      CumError = 0;
      LastActual = actual;
      Start = TRUE;
      StartCount = 0;
      if ((actual - expected) < 0)
        Positive = FALSE;
      else
        Positive = TRUE;
    }
    if (StartCount > STARTMIN)
    {
      Start = FALSE; // Start accumulating error only when near target setpoint
    }
    else
      StartCount++;

    /* Proportional */
    /* Add correction proportional to the current error */
    error = (actual - expected);
    p = error * (int)GKP;

    /* Integral */
    /* Correct accumulated error */
    /* Check for runaway accumulated error */
    if (error == 0) // If we hit target setpoint exactly then cut all power
      CumError = 0;
    if (Positive & (error < 0)) // If we passed the target then disgard accumulated error
    {
      CumError = 0;
      Positive = FALSE;
    }
    if (!Positive & (error > 0)) // If we passed the target then disgard accumulated error
    {
      CumError = 0;
      Positive = TRUE;
    }
    i = CumError * (int)GKI / (int)GT; // p covers current error

    // Ignore initial stall condition before the robot begins to move
    // Don't accumulate unless we're stalled. This will then gradually overcome the stall
    if ((actual == LastActual) & !Start)
        CumError += error;

    if (CumError > (int)GCUMLIMIT) // Detect and avoid integral windup
      CumError = (int)GCUMLIMIT;
    if (CumError < (int)-GCUMLIMIT)// Detect and avoid integral windup
      CumError = (int)-GCUMLIMIT;

    /* Derivative */
    /* Sudden reaction to sudden changes in the error rate */
    d = (actual - LastActual) * (int)GKD;
    LastActual = actual;

//  pid = p;
  pid = p + i;
//  pid = p + i - d;
//  pid = p - d;

  return pid;
}
